from uuid import UUID

import fastapi
from fastapi.security import OAuth2PasswordRequestForm

from schemas import (
    UserAllOptionalSchema, CreateUserSchema, CreatedUserResponseSchema,
    UpdateUserSchema
)
from ctrl import UserController
import services


user_router = fastapi.APIRouter(prefix='/users')
auth_router = fastapi.APIRouter(prefix='/auth')
ctrl = UserController()


@user_router.get('')
async def get_users(query: UserAllOptionalSchema = fastapi.Depends()):
    return await ctrl.get_list(**query.model_dump(exclude_none=True))


@user_router.get('/{id}')
async def get_users(id: UUID):
    return await ctrl.get(id)


@user_router.post('')
async def create_users(
        body: CreateUserSchema
) -> CreatedUserResponseSchema:
    return await ctrl.create(**body.model_dump(exclude_none=True))


@user_router.patch('/{id}')
async def update_users(
    id: UUID, body: UpdateUserSchema,
) -> CreatedUserResponseSchema:
    return await ctrl.update(id, **body.model_dump(exclude_none=True))


@user_router.delete('/{id}')
async def delete_users(
    id: UUID,
):
    return await ctrl.delete(id)


@auth_router.post('/login')
async def login(
    form_data: OAuth2PasswordRequestForm = fastapi.Depends(),
):
    return await services.auth_service.login(form_data)


@auth_router.get('/users/me')
async def read_user_me(
    user: dict = fastapi.Depends(services.auth_service.get_current_user)
):
    return user


@auth_router.post('/refresh')
async def refresh_token(
    data: dict,
    user: dict = fastapi.Depends(services.auth_service.get_current_user)
):
    return services.token_service.refresh_token(data['refresh_token'])
